local assets=
{
    Asset("ANIM", "anim/olaf_snowballpole_c_build.zip"),

    Asset("ATLAS", "images/inventoryimages/olaf_eyebone.xml"),
    Asset("IMAGE", "images/inventoryimages/olaf_eyebone.tex"),
}

local SPAWN_DIST = 30

local function GetSpawnPoint(pt)
    local theta = math.random() * 2 * PI
    local radius = SPAWN_DIST
    local offset = FindWalkableOffset(pt, theta, radius, 12, true)
    return offset ~= nil and (pt + offset) or nil
end

local function SpawnOlaf(inst)
    --print("chester_eyebone - SpawnChester")
    local pt = inst:GetPosition()
    --print("    near", pt)
        
    local spawn_pt = GetSpawnPoint(pt)
    if spawn_pt ~= nil then
        --print("    at", spawn_pt)
        local Olaf = SpawnPrefab("olaf_c")
        if Olaf ~= nil then
            Olaf.Physics:Teleport(spawn_pt:Get())
            Olaf:FacePoint(pt:Get())

            return Olaf
        end

    --else
        -- this is not fatal, they can try again in a new location by picking up the bone again
        --print("chester_eyebone - SpawnChester: Couldn't find a suitable spawn point for chester")
    end
end

local StartRespawn

local function StopRespawn(inst)
    --print("chester_eyebone - StopRespawn")
    if inst.respawntask ~= nil then
        inst.respawntask:Cancel()
        inst.respawntask = nil
        inst.respawntime = nil
    end
end

local function RebindOlaf(inst, Olaf)
    Olaf = Olaf or TheSim:FindFirstEntityWithTag("olaf_c")
    if Olaf ~= nil then
        inst.AnimState:PlayAnimation("idle_loop", true)
        inst:ListenForEvent("death", function() StartRespawn(inst, TUNING.CHESTER_RESPAWN_TIME) end, Olaf)

        if Olaf.components.follower.leader ~= inst then
            Olaf.components.follower:SetLeader(inst)
        end
        return true
    end
end

local function RespawnOlaf(inst)
    --print("chester_eyebone - RespawnChester")
    StopRespawn(inst)
    RebindOlaf(inst, TheSim:FindFirstEntityWithTag("olaf_c") or SpawnOlaf(inst))
end

StartRespawn = function(inst, time)
    StopRespawn(inst)

    time = time or 0
    inst.respawntask = inst:DoTaskInTime(time, RespawnOlaf)
    inst.respawntime = GetTime() + time
    inst.AnimState:PlayAnimation("dead", true)
end

local function FixOlaf(inst)
	inst.fixtask = nil
	--take an existing chester if there is one
	if not RebindOlaf(inst) then
        inst.AnimState:PlayAnimation("dead", true)
		
		if inst.components.inventoryitem.owner ~= nil then
			local time_remaining = 0
			local time = GetTime()
			if inst.respawntime and inst.respawntime > time then
				time_remaining = inst.respawntime - time		
			end
			StartRespawn(inst, time_remaining)
		end
	end
end

local function OnPutInInventory(inst)
    if inst.fixtask == nil then
        inst.fixtask = inst:DoTaskInTime(1, FixOlaf)
    end
end

local function OnSave(inst, data)
    --print("chester_eyebone - OnSave")
    if inst.respawntime ~= nil then
        local time = GetTime()
        if inst.respawntime > time then
            data.respawntimeremaining = inst.respawntime - time
        end
    end
end

local function OnLoad(inst, data)
    if data == nil then
        return
    end

    if data.respawntimeremaining ~= nil then
        inst.respawntime = data.respawntimeremaining + GetTime()
    end
end

local function GetStatus(inst)
    --print("smallbird - GetStatus")
    if inst.respawntask ~= nil then
        return "WAITING"
    end
end


local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddNetwork()
	inst.entity:AddMiniMapEntity()
	inst.MiniMapEntity:SetIcon("olaf_eyebone.tex")

    --so I can find the thing while testing
    --inst.entity:AddMiniMapEntity()
    --inst.MiniMapEntity:SetIcon("treasure.png")

    MakeInventoryPhysics(inst)

    inst:AddTag("olaf_snowballpole")
    inst:AddTag("irreplaceable")
    inst:AddTag("nonpotatable")
 	inst:AddTag("frozen")

    inst.AnimState:SetBank("snowballpole_c")
    inst.AnimState:SetBuild("olaf_snowballpole_c_build")
    inst.AnimState:PlayAnimation("idle_loop", true)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("inventoryitem")
    --inst.components.inventoryitem:SetOnPutInInventoryFn(OnPutInInventory)
    inst.components.inventoryitem.imagename = "olaf_eyebone"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/olaf_eyebone.xml"

    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = GetStatus
    inst.components.inspectable:RecordViews()
	
	inst:AddComponent("leader")

    MakeHauntableLaunch(inst)

    inst.OnLoad = OnLoad
    inst.OnSave = OnSave
	
	inst.fixtask = inst:DoTaskInTime(1, FixOlaf)

    return inst
end

return Prefab("common/inventory/olaf_eyebone", fn, assets)